@class Shape, List, NXDataLinkManager;

@interface Camera : N3DMovieCamera
{ 
    int currSurface;		// to remember it after wireframe temp
    int fidelity;
    float shadingRate;
    float worldScale;
    float grid;
    RtColor selectColor;
    RtColor unselectColor;
    
    struct _caFlags {
	unsigned int trackingUserAction:1; // conditional used in drawPS::
	unsigned int drawSelectionMarks:1; // conditional used in drawPS::
	unsigned int dontDrawSlaves:1;
	unsigned int cropToSelectionRect:1;  // for rendering a selection
	unsigned int projectionPosition:3;
	unsigned int gridActive:1;
	unsigned int gridVisible:1;
	unsigned int allSelected:1;
	unsigned int useUnSelectColor:1;
	unsigned int locationNumber:5;		// max of 32 [that's alot]
	unsigned int ignoreFiles:1;
    } caFlags;
    
    NXDataLinkManager *linkManager;	/* manager of data links */
    
    id constraintsManager;
    id tracker;
    
    id cursor;
    
    List *locationList;
    id image;
    NXPoint imageOrigin;
    id _private2;
    NXRect _selectRect;
}

+ currentCamera;

- initFrame: (const NXRect *) frameRect;

- setSelectColor;
- setUnSelectColor; // does not use caFlags.useUnSelectColor

- (BOOL)useUnSelectColor;
- setUseUnSelectColor:(BOOL)flag;

- (Shape *)addObject:newObject;
- connectShape:(Shape *)shape to:(Shape *)connectShape;
- connectShape:(Shape *)shape;
- connectShapeAndDisplay:shape;

- selectAll:sender;
- deselectAll:sender;

- (Shape *)selectedShape;
- (Shape *)anySelectedShape;

- selectShape:(Shape *)shape
    andDisplay:(BOOL)dFlag
    updateBrowser:(BOOL)hFlag
    updateInspector:(BOOL)iFlag;
- selectShape:(Shape *)shape andDisplay:(BOOL)flag updateBrowser:(BOOL)hFlag;
- selectShape:(Shape *)shape andDisplay:(BOOL)flag;
- selectShape:(Shape *) aShape;

- getSelectedShapes:(List *) sList;
- setSelectedShapes:(List *) sList;

- moveShape:(Shape *)aShape to:(NXPoint *)p;

- setRenderTypeTemporarily:(int)surface shape:(Shape *)shape;
- setRenderTypeTemporarily:(int)surface;
- restoreRenderType;

- updateBrowserAndInspector;
- updateBrowser;
- updateInspector;

- deleteShape:shape;

+ writeShape:(Shape *)shape toStream:(NXStream *)stream;
+ (Shape *)readShapeFromStream:(NXStream *)stream;

- drawSlaves;
- displayOnlySelf;

- setCursor:aCursor;

- (Shape *)shapeAt:(const NXPoint *)viewPoint;

- (int) shapeCount;
- (int) selectedCount;

- (int) renderAsTIFF;
- (int) renderAsTIFFFor:anObj;
- (int) renderAsEPS;
- (int) renderAsEPSFor:anObj;

- dirty:sender;
@end