@class Shader, Camera;

@interface Shape:N3DShape
{
@public
    struct {
	unsigned int isSolid:1;
	unsigned int isSelected:1;
	unsigned int isEditable:1;
	unsigned int isOriginCentered:1;
	unsigned int dontDraw:1;
	unsigned int padding:27;
    } shFlags;
@private    
    int identifier;
    id _private1;
    id _private2;
}

// same functionality as renderSelf: from N3DShape
- doRenderSelf: (Camera *) camera;

// draw the standard x-y-z at the origin of the shape's coordinate system
- drawAxis:(Camera *)camera;

// resets the shape to its default position/scale/rotation, usually puts
// the center or lower-left-front corner of the shape's bounding volume
// at the origin of the shape's coordinate system
- resetPosition;

// set/query selection status
- setSelected:(BOOL)flag;
- (BOOL) isSelected;

- setSolid:(BOOL)flag;
- (BOOL)isSolid;

- setDoDraw:(BOOL)flag;
- (BOOL)doesDraw;

- (BOOL) goodBounds;

// link aShape as last peer of self
- linkAsLastPeer:(Shape *)aShape;

// calculate default bounding box
- calcBoundingBox;

- getBounds:(NXRect *)sRect inCamera:camera;

// cover for class method
- iconImage;
// returns the NXImage for this shape
+ iconImage;

- makeDescendantsPerform:(SEL)action;
- makeDescendantsPerform:(SEL)action with:arg;

- wasRemovedFrom:(Camera *)sender;
- wasAddedTo:(Camera *)sender;

- read:(NXTypedStream *)stream;
- write:(NXTypedStream *)stream;
- awake;

- awakeInCamera:(Camera *) aCamera;
@end

// override these to get some notification
@interface Shape (NotificationMethods)
// translation notification
- willPreTranslate:(float *)x :(float *)y :(float *)z;
- didPreTranslate:(float *)x :(float *)y :(float *)z;
- willTranslate:(float *)x :(float *)y :(float *)z;
- didTranslate:(float *)x :(float *)y :(float *)z;
- willPlaceAt:(float *)x :(float *)y :(float *)z;
- didPlaceAt:(float *)x :(float *)y :(float *)z;

// direct matrix manipulation
- willConcatTransformMatrix:(RtMatrix)tm premultiply:(BOOL *)flag;
- didConcatTransformMatrix:(RtMatrix)tm premultiply:(BOOL)flag;
- willSetTransformMatrix:(RtMatrix)tm;
- didSetTransformMatrix:(RtMatrix)tm;

// rotation notification (N3DRotator uses the above)
- willPreRotateAngle:(float *)num axis:(float *)axis;
- didPreRotateAngle:(float *)num axis:(float *)axis;

// scale notification
- willPreScale:(float *)x :(float *)y :(float *)z;
- didPreScale:(float *)x :(float *)y :(float *)z;
- willScale:(float *)x :(float *)y :(float *)z;
- didScale:(float *)x :(float *)y :(float *)z;
- willScaleUniformly:(float *)s;
- didScaleUniformly:(float) s;
- willPreScaleUniformly:(float *)s;
- didPreScaleUniformly:(float) s;

// hierarchy notification
- willLinkPeer:(id *)aPeer;
- didLinkPeer:aPeer;
- willLinkDescendant:(id *)aDescendant;
- didLinkDescendant:aDescendant;
- willUnlink;
- didUnlink;

// name notification
- willSetShapeName:(const char **)aName;
- didSetShapeName:(const char *)aName;
@end
